/*
 * Copyright (c) 2011-2022 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef REGEX_COMPILER_H_
#define REGEX_COMPILER_H_
#include <sys/types.h>
#include <stdint.h>
#include "regex_rof.h"

#define REGEX_COMPILER_VERSION "22.10.2"

/***************************************
 * ENUMS
 ***************************************/
enum regex_compiler_objective
{
    REGEX_COMPILER_OBJECTIVE_DEFAULT = 0,
    REGEX_COMPILER_OBJECTIVE_1 = 1,
    REGEX_COMPILER_OBJECTIVE_2 = 2,
    REGEX_COMPILER_OBJECTIVE_3 = 3,
    REGEX_COMPILER_OBJECTIVE_4 = 4,
    REGEX_COMPILER_OBJECTIVE_5 = 5
};

enum regex_compiler_options_flags
{
    REGEX_COMPILER_OPTIONS_AUTO_RULE_ID = 1,
    REGEX_COMPILER_OPTIONS_CHECKSUM = 2,
    REGEX_COMPILER_OPTIONS_EM = 4,
    REGEX_COMPILER_OPTIONS_FORCE = 8,
    REGEX_COMPILER_OPTIONS_INCREMENTAL = 16,
    REGEX_COMPILER_OPTIONS_NO_INC_COMPILE_PADDING = 32,
    REGEX_COMPILER_OPTIONS_PCRE_PRE_8_36 = 64,
    REGEX_COMPILER_OPTIONS_STRICT_QUANTIFIERS = 128,
    REGEX_COMPILER_OPTIONS_QUICK_REMOVE_RULES = 256,
    REGEX_COMPILER_OPTIONS_QUICK_ADD_RULES = 512,
    REGEX_COMPILER_OPTIONS_UTF_8 = 1024,
    REGEX_COMPILER_OPTIONS_DISABLE_BIDIRECTIONAL = 2048,
    REGEX_COMPILER_OPTIONS_UTF_16_STRING = 4096,
    REGEX_COMPILER_OPTIONS_SWITCH_OFF_LOCALE_SUPPORT = 8192,
    REGEX_COMPILER_OPTIONS_SPLIT_ALTERNATIONS = 16384,
    REGEX_COMPILER_OPTIONS_FORCE_EARLY_SSID_CHECK = 32768,
    REGEX_COMPILER_OPTIONS_SUB_PATTERN = 65536,
    REGEX_COMPILER_OPTIONS_NO_SUBSET_CROSS = 131072
};

enum regex_compiler_output_flags
{
    REGEX_COMPILER_OUTPUT_COMPILATION_STATISTICS = 1,
    REGEX_COMPILER_OUTPUT_COMPILED_RULES_LOG = 2,
    REGEX_COMPILER_OUTPUT_CRITICAL_RULES_RANK = 4,
    REGEX_COMPILER_OUTPUT_RULE_DIRECTION_ANALYSIS = 8,
    REGEX_COMPILER_OUTPUT_UNCOMPILED_RULES_LOG = 16,
    REGEX_COMPILER_OUTPUT_ROFF = 32,
    REGEX_COMPILER_OUTPUT_TYPE_8=64
};

enum regex_error_codes
{
    REGEX_ERROR_NONE = 0,
    REGEX_ERROR_SYNTAX = 1,
    REGEX_ERROR_STARTS_WITH_CLASS = 2,
    REGEX_ERROR_STARTS_WITH_ONE_BYTE_UNANCHORED_PREFIX = 3,
    REGEX_ERROR_INSUFFICIENT_PREFIX_RESOURCES = 4,
    REGEX_ERROR_INSUFFICIENT_THREAD_INSTRUCTION_RESOURCES = 5,
    REGEX_ERROR_INSUFFICIENT_THREAD_CLASS_RESOURCES = 6,
    REGEX_ERROR_INSUFFICIENT_REGISTER_RESOURCES = 7,
    REGEX_ERROR_NO_VALID_PREFIX_FOUND = 8,
    REGEX_ERROR_PREFIX_FANOUT_COUNT_EXCEEDED = 9,
    REGEX_ERROR_PREFIX_OVERFLOW = 10,
    REGEX_ERROR_OUT_OF_CLASS_IDS = 11,
    REGEX_ERROR_PTPB_THRESHOLD_EXCEEDED = 12,
    REGEX_ERROR_BRANCH_LIMIT_EXCEEDED = 13,
    REGEX_ERROR_AMBIGUOUS_REPETITION_WITH_MULTILINE = 14,
    REGEX_ERROR_SANITY_BIDIRECTIONAL = 15,
    REGEX_ERROR_SANITY = 16
};

enum regex_global_regex_options_flags
{
    REGEX_GLOBAL_REGEX_OPTIONS_CASELESS = 1, /*Switch on global case insensitivity*/
    REGEX_GLOBAL_REGEX_OPTIONS_MULTILINE = 2, /*Switch on global multiline mode*/
    REGEX_GLOBAL_REGEX_OPTIONS_NO_SINGLE = 4, /*Switch off global single-line mode*/
    REGEX_GLOBAL_REGEX_OPTIONS_XML = 8, /*Switch on XML schema mode*/
    REGEX_GLOBAL_REGEX_OPTIONS_FREE_SPACING = 16 /*Switch on global free spacing mode*/
};

enum regex_prefix_capacity
{
    REGEX_PREFIX_CAPACITY_2K = 0,
    REGEX_PREFIX_CAPACITY_4K = 1,
    REGEX_PREFIX_CAPACITY_8K = 2,
    REGEX_PREFIX_CAPACITY_16K = 3,
    REGEX_PREFIX_CAPACITY_32K = 4,
    REGEX_PREFIX_CAPACITY_64K = 5
};


enum regex_prefix_selection_control_list_entry_type
{
    REGEX_PREFIX_SELECTION_CONTROL_LIST_ENTRY_DENYLIST = 0,
    REGEX_PREFIX_SELECTION_CONTROL_LIST_ENTRY_GRAYLIST = 1,
    REGEX_PREFIX_SELECTION_CONTROL_LIST_ENTRY_ALLOWLIST = 2,
    REGEX_PREFIX_SELECTION_CONTROL_LIST_ENTRY_PREDEFINED_AS_DENYLIST = 3,
    REGEX_PREFIX_SELECTION_CONTROL_LIST_ENTRY_PREDEFINED_AS_GRAYLIST = 4,
    REGEX_PREFIX_SELECTION_CONTROL_LIST_ENTRY_PREDEFINED_AS_ALLOWLIST = 5
};

enum regex_rof_entry_type
{
    REGEX_ROF_LEGACY_RULES_MEMORY = 0,
    REGEX_ROF_ENTRY_CHECK_CSR = 1,
    REGEX_ROF_ENTRY_CHECK_CSR_GTE = 2,
    REGEX_ROF_ENTRY_CHECK_CSR_LTE = 3,
    REGEX_ROF_ENTRY_CHECK_CSR_CHECKSUM = 4,
    REGEX_ROF_ENTRY_CHECK_CSR_CHECKSUM_EXCLUDING_EM = 5,
    REGEX_ROF_ENTRY_IM_INSTRUCTION = 6,
    REGEX_ROF_ENTRY_EM_INSTRUCTION = 7,
    REGEX_ROF_ENTRY_EM_BLOCK = 8
};

enum regex_rule_direction_type
{
    REGEX_RULE_DIRECTION_TYPE_UP = 0,
    REGEX_RULE_DIRECTION_TYPE_DOWN = 1,
    REGEX_RULE_DIRECTION_TYPE_UP_DOWN = 2,
    REGEX_RULE_DIRECTION_TYPE_DOWN_UP = 3,
    REGEX_RULE_DIRECTION_TYPE_VIRTUAL_UP = 4,/* For virtual prefixes */
    REGEX_RULE_DIRECTION_TYPE_NONE = 5/* For cases when no rule direction is specified */
};

enum regex_status_codes
{
    REGEX_STATUS_OK = 0, /*Everything is fine*/
    REGEX_STATUS_NO_RULES_FOUND = 1, /*No rules have been found*/
    REGEX_STATUS_SYNTAX_ERROR = 2, /*A syntax error was found, see uncompiled rules log*/
    REGEX_STATUS_FATAL_ERROR = 3 /*Compilation was aborted due to a fatal error*/
};

enum regex_tpe_data_width
{
    REGEX_TPE_DATA_WIDTH_DEFAULT = 0,
    REGEX_TPE_DATA_WIDTH_1 = 1,
    REGEX_TPE_DATA_WIDTH_2 = 2,
    REGEX_TPE_DATA_WIDTH_3 = 3,
    REGEX_TPE_DATA_WIDTH_4 = 4,
    REGEX_TPE_DATA_WIDTH_5 = 5,
    REGEX_TPE_DATA_WIDTH_6 = 6,
    REGEX_TPE_DATA_WIDTH_7 = 7,
    REGEX_TPE_DATA_WIDTH_8 = 8
};

enum regex_verbose_level
{
    REGEX_VERBOSE_LEVEL_0 = 0, /*No terminal output*/
    REGEX_VERBOSE_LEVEL_1 = 1, /*Only warnings and errors*/
    REGEX_VERBOSE_LEVEL_2 = 2, /*All output except progress bars*/
    REGEX_VERBOSE_LEVEL_3 = 3 /*All output*/
};

enum regex_virtual_prefix_mode
{
    /*Only attempt to use virtual prefix if:
      VIRTUAL keyword specified per rule*/
    REGEX_VIRTUAL_PREFIX_MODE_0 = 0,
    /*Only attempt to use virtual prefix if:
      VIRTUAL keyword specified per rule
      or a valid prefix cannot be extracted for that rule*/
    REGEX_VIRTUAL_PREFIX_MODE_1 = 1,
    /*Only attempt to use virtual prefix if:
      VIRTUAL keyword specified per rule
      or a prefix cannot be mapped to the rules memories*/
    REGEX_VIRTUAL_PREFIX_MODE_2 = 2,
    /*Only attempt to use virtual prefix if:
      VIRTUAL keyword specified per rule
      or a valid prefix cannot be extracted for that rule
      or a prefix cannot be mapped to the rules memories*/
    REGEX_VIRTUAL_PREFIX_MODE_3 = 3
};

enum regex_version
{
    REGEX_VERSION_V5_7 = 0,
    REGEX_VERSION_V6_0 = 2
};

/***************************************
 * STRUCTS
 ***************************************/
struct regex_rule_entry
{
        uint32_t number_of_prefix_entries;
        char **prefix;
        enum regex_rule_direction_type rule_direction_type;
        uint32_t subset_id;
        uint32_t rule_id;
        char *rule;
        int32_t gvmi_id;
        int32_t pd_id;
};

struct regex_compiled_rules_log_entry
{
        uint32_t regex_rule_id;
        enum regex_rule_direction_type rule_direction_type;
        struct regex_rule_entry rule;
};

struct regex_uncompiled_rules_log_entry
{
        uint32_t regex_rule_id;
        struct regex_rule_entry rule;
        enum regex_error_codes error;
        char *message;
        char *location;
};

struct regex_critical_rules_rank_entry
{
        uint32_t rule_rank;
        uint32_t ruleset_rank;
        uint32_t num_anchored_one_byte_prefixes;
        uint32_t num_anchored_two_byte_prefixes;
        uint32_t num_anchored_three_byte_prefixes;
        uint32_t num_anchored_four_byte_prefixes;
        uint32_t average_thread_count_per_anchored_prefix;
        uint32_t num_unanchored_one_byte_prefixes;
        uint32_t num_unanchored_two_byte_prefixes;
        uint32_t num_unanchored_three_byte_prefixes;
        uint32_t num_unanchored_four_byte_prefixes;
        uint32_t average_thread_count_per_unanchored_prefix;
        float primary_threads_per_byte;
        struct regex_compiled_rules_log_entry compiled_rules_log_entry;
};

struct regex_uncompiled_rules_log
{
        uint32_t number_of_entries;
        struct regex_uncompiled_rules_log_entry *uncompiled_rules_log_entries;
};

struct regex_compilation_statistics
{
        uint32_t total_number_of_rules;
        uint32_t rules_compiled;
        uint32_t one_byte_prefixes;
        uint32_t two_byte_prefixes;
        uint32_t three_byte_prefixes;
        uint32_t four_byte_prefixes;
        uint32_t instrucions_in_tcm;
        uint32_t instrucions_in_em;
        uint32_t class_ram_entries_used;
};

struct regex_compiled_rules_log
{
        uint32_t number_of_entries;
        struct regex_compiled_rules_log_entry *compiled_rules_log_entries;
};

struct regex_critical_rules_rank
{
        uint32_t number_of_entries;
        struct regex_critical_rules_rank_entry *critical_rules_rank_entry;
};

struct regex_prefix_selection_control_list_entry
{
        enum regex_prefix_selection_control_list_entry_type type;
        char *value;
};

struct regex_prefix_selection_control_list
{
        uint32_t number_of_entries;
        struct regex_prefix_selection_control_list_entry *prefix_selection_control_list_entries;
};

struct regex_denylist_data_sample
{
        float threshold;
        size_t length;
        char *data;
};

struct regex_roff
{
        size_t length;
        char *data;
};

struct regex_ruleset
{
        uint32_t number_of_entries;
        struct regex_rule_entry *rules;
};

struct regex_rule_ids_to_remove
{
        uint32_t number_of_entries;
        uint32_t *list_of_rule_ids_to_remove;
};

/***************************************
 * C++ FUNCTION PROTOTYPES
 ***************************************/
#ifdef __cplusplus
class regex_compiler
{
    public:
        regex_compiler();
        virtual ~regex_compiler();

        uint32_t regex_compile(
                /*INPUT*/
                regex_ruleset *ruleset,
                regex_prefix_selection_control_list *prefix_selection_control_list,

                /*OPTIONS*/
                /*regex_compiler_options_flags*/ uint32_t compiler_options,
                /*regex_global_regex_options_flags*/ uint32_t global_regex_options,
                uint32_t threads,
                regex_version version,
                regex_verbose_level verbose,
                /* regex_compiler_output_flags*/ uint32_t compiler_output_flags,

                /*OUTPUT*/
                regex_compilation_statistics **compilation_statistics,
                regex_compiled_rules_log **compiled_rules_log,
                regex_critical_rules_rank **critical_rules_rank,
                regex_uncompiled_rules_log **uncompiled_rules_log,
                regex_rof **rof,
                regex_roff **roff);

        uint32_t regex_compile_advanced(
                /*INPUT*/
                regex_ruleset *ruleset,
                regex_prefix_selection_control_list *prefix_selection_control_list,
                regex_rof *rof_for_incremental_compile,
                regex_roff *roff_for_incremental_compile,
                regex_rof *rofi_for_incremental_compile,
                regex_rule_ids_to_remove *rule_ids_to_remove,
                regex_denylist_data_sample *denylist_sample_data,

                /*OPTIONS*/
                /*regex_compiler_options_flags*/ uint32_t compiler_options,
                regex_prefix_capacity prefix_capacity,
                /*regex_global_regex_options_flags*/ uint32_t global_regex_options,
                float set_auto_denylist,
                uint32_t max,
                regex_compiler_objective objective,
                float ptpb_threshold,
                uint32_t max_rep_max,
                uint32_t threads,
                regex_version version,
                regex_verbose_level verbose,
                regex_tpe_data_width tpe_data_width,
                regex_virtual_prefix_mode virtual_prefix_mode,
                /* regex_compiler_output_flags*/ uint32_t compiler_output_flags,
                uint32_t divide_ruleset,

                /*OUTPUT*/
                regex_compilation_statistics **compilation_statistics,
                regex_compiled_rules_log **compiled_rules_log,
                regex_critical_rules_rank **critical_rules_rank,
                regex_uncompiled_rules_log **uncompiled_rules_log,
                regex_rof **rof,
                regex_roff **roff,
                regex_rof **rofi,
                regex_prefix_selection_control_list **generated_pscl);

        int read_denylist_sample_data_file(char *denylist_sample_data_filename,
                regex_denylist_data_sample *denylist_sample_data);

        int read_remove_ruleset_file(char *remove_ruleset_filename,
                regex_rule_ids_to_remove *rule_ids_to_remove);

        int read_prefix_selection_control_list_file(char *prefix_selection_control_list_filename,
                regex_prefix_selection_control_list *prefix_selection_control_list);

        int read_ruleset_file(char *ruleset_filename,
                regex_ruleset *ruleset,
                char **ruleset_errors);

        int read_roff_file(char *roff_filename,
                regex_roff *roff);

        int read_roff_file_bluefield(char *roff_filename,
                regex_roff *roff,
                regex_version version);

        int read_rof_file(char *rof_filename,
                regex_rof *rof);

        int read_rof_file_bluefield(char *rof_filename,
                regex_rof *rof,
                regex_version version);

        int read_binary_rof_file(char *rof_filename,
                regex_rof *rof);

        int read_binary_rof_file_bluefield(char *rof_filename,
                regex_rof *rof,
                regex_version version);

        int generate_binary_rof(char **binary_rof,
                uint64_t *binary_rof_len,
                const regex_rof *rof);

        int write_rule_rank(char *rule_rank_filename,
                regex_critical_rules_rank *critical_rules_rank);

        int write_rule_direction_analysis(char *rule_direction_analysis_filename,
                regex_compiled_rules_log *compiled_rules_log);

        int write_uncompiled_rules_log(char *uncompiled_rules_log_filename,
                regex_uncompiled_rules_log *uncompiled_rules_log);

        int write_uncompiled_rules_summary(char *uncompiled_rules_summary_filename,
                regex_uncompiled_rules_log *uncompiled_rules_log);

        int write_ruleset_errors_log(char *uncompiled_rules_log_filename,
                char *ruleset_errors);

        int write_rule_id_lookup_table(char *rule_id_lookup_table_filename,
                regex_compiled_rules_log *compiled_rules_log);

        int write_roff_file(char *roff_filename,
                regex_roff *roff);

        int write_rof_file(char *rof_filename,
                regex_rof *rof);

        int merge_rof_file(char *rof_filename_output,
                char *rof_filename_1,
                char *rof_filename_2);

        int merge_binary_rof_file(char *rof_filename_output,
                char *rof_filename_1,
                char *rof_filename_2);

        int merge_roff_file(char *roff_filename_output,
                char *roff_filename_1,
                char *roff_filename_2);

        int write_prefix_selection_control_list(char *prefix_selection_control_list_filename,
                regex_prefix_selection_control_list *generated_pscl);

        int free_structs(regex_compilation_statistics **compilation_statistics,
                regex_compiled_rules_log **compiled_rules_log,
                regex_critical_rules_rank **critical_rules_rank,
                regex_uncompiled_rules_log **uncompiled_rules_log,
                regex_rof **rof,
                regex_roff **roff,
                regex_rof **rofi,
                regex_prefix_selection_control_list **generated_pscl,
                uint32_t divide_ruleset);
};
#endif /* __cplusplus */

/***************************************
 * C FUNCTION PROTOTYPES
 ***************************************/
#ifdef __cplusplus
extern "C" {
#endif
    int32_t regex_compile(
            /*INPUT*/
            struct regex_ruleset *ruleset,
            struct regex_prefix_selection_control_list *prefix_selection_control_list,

            /*OPTIONS*/
            /*regex_compiler_options_flags*/ uint32_t compiler_options,
            /*regex_global_regex_options_flags*/ uint32_t global_regex_options,
            uint32_t threads,
            enum regex_version version,
            enum regex_verbose_level verbose,
            /* regex_compiler_output_flags*/ uint32_t compiler_output_flags,

            /*OUTPUT*/
            struct regex_compilation_statistics **compilation_statistics,
            struct regex_compiled_rules_log **compiled_rules_log,
            struct regex_critical_rules_rank **critical_rules_rank,
            struct regex_uncompiled_rules_log **uncompiled_rules_log,
            struct regex_rof **rof,
            struct regex_roff **roff);

    int32_t regex_compile_advanced(
            /*INPUT*/
            struct regex_ruleset *ruleset,
            struct regex_prefix_selection_control_list *prefix_selection_control_list,
            struct regex_rof *rof_for_incremental_compile,
            struct regex_roff *roff_for_incremental_compile,
            struct regex_rof *rofi_for_incremental_compile,
            struct regex_rule_ids_to_remove *rule_ids_to_remove,
            struct regex_denylist_data_sample *denylist_sample_data,

            /*OPTIONS*/
            /*regex_compiler_options_flags*/ uint32_t compiler_options,
            enum regex_prefix_capacity prefix_capacity,
            /*regex_global_regex_options_flags*/ uint32_t global_regex_options,
            float set_auto_denylist,
            uint32_t max,
            enum regex_compiler_objective objective,
            float ptpb_threshold,
            uint32_t max_rep_max,
            uint32_t threads,
            enum regex_version version,
            enum regex_verbose_level verbose,
            enum regex_tpe_data_width tpe_data_width,
            enum regex_virtual_prefix_mode virtual_prefix_mode,
            /* regex_compiler_output_flags*/ uint32_t compiler_output_flags,
            uint32_t divide_ruleset,

            /*OUTPUT*/
            struct regex_compilation_statistics **compilation_statistics,
            struct regex_compiled_rules_log **compiled_rules_log,
            struct regex_critical_rules_rank **critical_rules_rank,
            struct regex_uncompiled_rules_log **uncompiled_rules_log,
            struct regex_rof **rof,
            struct regex_roff **roff,
            struct regex_rof **rofi,
            struct regex_prefix_selection_control_list **generated_pscl);

    uint32_t regex_read_denylist_sample_data_file(char *denylist_sample_data_filename,
            struct regex_denylist_data_sample *denylist_sample_data);

    uint32_t regex_read_remove_ruleset_file(char *remove_ruleset_filename,
            struct regex_rule_ids_to_remove *rule_ids_to_remove);

    uint32_t regex_read_prefix_selection_control_list_file(char *prefix_selection_control_list_filename,
            struct regex_prefix_selection_control_list *prefix_selection_control_list);

    uint32_t regex_read_ruleset_file(char *ruleset_filename,
            struct regex_ruleset *ruleset,
            char **ruleset_errors);

    uint32_t regex_read_roff_file(char *roff_filename,
            struct regex_roff *roff);

    uint32_t regex_read_roff_file_bluefield(char *roff_filename,
            struct regex_roff *roff,
            enum regex_version version);

    uint32_t regex_read_rof_file(char *rof_filename,
            struct regex_rof *rof);

    uint32_t regex_read_rof_file_bluefield(char *rof_filename,
            struct regex_rof *rof,
            enum regex_version version);

    uint32_t regex_read_binary_rof_file(char *rof_filename,
            struct regex_rof *rof);

    uint32_t regex_read_binary_rof_file_bluefield(char *rof_filename,
            struct regex_rof *rof,
            enum regex_version version);

    uint32_t regex_generate_binary_rof(char **binary_rof,
            uint64_t *binary_rof_len,
            const struct regex_rof *rof);

    uint32_t regex_write_rule_rank(char *rule_rank_filename,
            struct regex_critical_rules_rank *critical_rules_rank);

    uint32_t regex_write_rule_direction_analysis(char *rule_direction_analysis_filename,
            struct regex_compiled_rules_log *compiled_rules_log);

    uint32_t regex_write_uncompiled_rules_log(char *uncompiled_rules_log_filename,
            struct regex_uncompiled_rules_log *uncompiled_rules_log);

    uint32_t regex_write_uncompiled_rules_summary(char *uncompiled_rules_summary_filename,
            struct regex_uncompiled_rules_log *uncompiled_rules_log);

    uint32_t regex_write_ruleset_errors_log(char *uncompiled_rules_log_filename,
            char *ruleset_errors);

    uint32_t regex_write_rule_id_lookup_table(char *rule_id_lookup_table_filename,
            struct regex_compiled_rules_log *compiled_rules_log);

    uint32_t regex_write_prefix_selection_control_list(char *prefix_selection_control_list_filename,
            struct regex_prefix_selection_control_list *generated_pscl);

    uint32_t regex_write_roff_file(char *roff_filename,
            struct regex_roff *roff);

    uint32_t regex_write_rof_file(char *rof_filename,
            struct regex_rof *rof);

    uint32_t regex_merge_rof_file(char *rof_filename_output,
            char *rof_filename_1,
            char *rof_filename_2);

    uint32_t regex_merge_binary_rof_file(char *rof_filename_output,
            char *rof_filename_1,
            char *rof_filename_2);

    uint32_t regex_merge_roff_file(char *roff_filename_output,
            char *roff_filename_1,
            char *roff_filename_2);

    uint32_t regex_free_structs(struct regex_compilation_statistics **compilation_statistics,
            struct regex_compiled_rules_log **compiled_rules_log,
            struct regex_critical_rules_rank **critical_rules_rank,
            struct regex_uncompiled_rules_log **uncompiled_rules_log,
            struct regex_rof **rof,
            struct regex_roff **roff,
            struct regex_rof **rofi,
            struct regex_prefix_selection_control_list **generated_pscl,
            uint32_t divide_ruleset);

#ifdef __cplusplus
}
#endif
#endif /* REGEX_COMPILER_H_ */
